<?php

namespace KPIs\Customer\Cron;

use KPIs\Customer\Logger\Logger;
/**
 * FileName  ResetCustomerAccess.php.
 * @project: skshop-magento2
 * @developer: Larisa Groza
 * @company: BitStone
 */

class ResetCustomerAccess
{
    /**
     * @var \Magento\Customer\Model\ResourceModel\Customer\CollectionFactory
     */
    protected $customerCollectionFactory;

    /**
     * @var \KPIs\Customer\Helper\Data
     */
    protected $helper;

    /**
     * @var Logger
     */
    protected $logger;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfigObject;

    /**
     * ResetCustomerAccess constructor.
     * @param \Magento\Customer\Model\ResourceModel\Customer\CollectionFactory $collectionFactory
     * @param \KPIs\Customer\Helper\Data $helper
     * @param Logger $logger
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfigObject
     */
    public function __construct(
        \Magento\Customer\Model\ResourceModel\Customer\CollectionFactory $collectionFactory,
        \KPIs\Customer\Helper\Data $helper,
        Logger $logger,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfigObject
    ) {
        $this->customerCollectionFactory = $collectionFactory;
        $this->helper = $helper;
        $this->logger = $logger;
        $this->scopeConfigObject = $scopeConfigObject;
    }

    /**
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function execute()
    {
        $customers = $this->customerCollectionFactory->create();
        $customers->addNameToSelect()
            ->addAttributeToSelect('customer_access',"left")
            ->addAttributeToSelect('valid_until',"left")
            ->addAttributeToSelect('customer_membership',"left")
            ->addFieldToFilter('valid_until', ['lt' => date('Y-m-d H:i:s')])
            ->addFieldToFilter('customer_access', ['gt' => 1])
            ->addFieldToFilter('customer_access', ['lt' => 8]);

        $allCustomerAccess = array_flip($this->helper->getAllCustomerAccess());
        $message = '\nCron - Reset Customer Access START';
        foreach ($customers as $customer) {
            $exCustomerAccess = strtolower($allCustomerAccess[$customer->getCustomerAccess()]);
            $newCustomerAccess = $this->_getCustomerAccess($allCustomerAccess, $exCustomerAccess);
            if ($newCustomerAccess) {
                $this->helper->updateKpiViewsAndAccess($customer->getId(), [
                    'customer_access'       => $newCustomerAccess,
                    'kpi_views_customer'    => $this->scopeConfigObject->getValue('kpi_views_sync/general/default_kpi_views'),
                ]);

                $message .= "\nCustomer with email {$customer->getEmail()} is expired from {$customer->getValidUntil()} and was changed from {$exCustomerAccess} to {$newCustomerAccess}";
            }
        }
        $message .= '\nCron - Reset Customer Access END';
        $this->logger->debug($message);
    }

    /**
     * Get new customer access.
     *
     * @param $customerAccess
     * @param $exCustomerAccess
     * @return bool|int
     */
    private function _getCustomerAccess($customerAccess, $exCustomerAccess)
    {
        for ($i = 9; $i <= 14; $i++) {
            $value = strtolower($customerAccess[$i]);
            if (strpos($value, $exCustomerAccess) !== FALSE) {
                return $i;
            }
        }
        return false;
    }
}