<?php
/**
 * ClassName  SetKpisViews.
 * @developer: Larisa Groza
 * @company: BitStone
 */

namespace KPIs\Customer\Observer;

use Magento\Framework\Stdlib\DateTime;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use KPIs\Customer\Logger\Logger;
use Magento\Framework\Registry;

class CustomerSaveAfter implements ObserverInterface
{
    /**
     * Logger of customer's log data.
     *
     * @var Logger
     */
    protected $logger;

    /**
     * @var \KPIs\Customer\Helper\Data
     */
    protected $helper;

    /**
     * @var Registry
     */
    protected $registry;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfigObject;

    /**
     * CustomerSaveAfter constructor.
     * @param \KPIs\Customer\Helper\Data $helper
     * @param Registry $registry
     * @param Logger $logger
     */
    public function __construct(
        \KPIs\Customer\Helper\Data $helper,
        Registry $registry,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfigObject,
        Logger $logger
    ) {
        $this->helper = $helper;
        $this->logger = $logger;
        $this->registry = $registry;
        $this->scopeConfigObject = $scopeConfigObject;
    }

    /**
     * Handler for 'customer_save_after' event triggered when log out
     *
     * @param Observer $observer
     * @return void
     */
    public function execute(Observer $observer)
    {
        $customer =  $observer->getCustomer();
        $customerId = $customer->getId();

        $kpiViews = $customer->getKpiViewsCustomer();
        $oldKpiViews = $this->registry->registry('kpi_views_customer_' . $customerId);
        $this->registry->unregister('kpi_views_customer_' . $customerId);

        $customerAccess = $customer->getCustomerAccess();
        $oldCustomerAccess = $this->registry->registry('customer_access_' . $customerId);
        $this->registry->unregister('customer_access_' . $customerId);

        if ($kpiViews != $oldKpiViews && abs($oldKpiViews-$kpiViews) > 1) {
            $this->helper->updateKpiViewsAndAccess($customerId, ['kpi_views_customer' => $kpiViews]);
        }
        if ($customerAccess != $oldCustomerAccess) {
            $this->helper->updateKpiViewsAndAccess($customerId, ['kpi_views_customer' => $kpiViews]);
        }
    }
}