<?php

namespace KPIs\Customer\Setup;

use Magento\Customer\Model\Customer;
use Magento\Customer\Setup\CustomerSetup;
use Magento\Customer\Setup\CustomerSetupFactory;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\UpgradeDataInterface;
use Magento\Eav\Model\Entity\Attribute\Set as AttributeSet;
use Magento\Eav\Model\Entity\Attribute\SetFactory as AttributeSetFactory;

/**
 * Kpi Sync - Upgrade Data.
 *
 * @category   Magento2
 * @author     Larisa Groza
 */
class UpgradeData implements UpgradeDataInterface
{
    /**
     * @var CustomerSetupFactory
     */
    private $customerSetupFactory;
    private $attributeSetFactory;

    public function __construct(CustomerSetupFactory $customerSetupFactory, AttributeSetFactory $attributeSetFactory)
    {
        $this->customerSetupFactory = $customerSetupFactory;
        $this->attributeSetFactory = $attributeSetFactory;
    }

    /**
     * Upgrades data for a module
     *
     * @param ModuleDataSetupInterface $setup
     * @param ModuleContextInterface $context
     * @return void
     */
    public function upgrade(ModuleDataSetupInterface $setup, ModuleContextInterface $context)
    {
        $dbVersion = $context->getVersion();
        $customerSetup = $this->customerSetupFactory->create(['setup' => $setup]);

        $customerEntity = $customerSetup->getEavConfig()->getEntityType('customer');
        $attributeSetId = $customerEntity->getDefaultAttributeSetId();

        /** @var $attributeSet AttributeSet */
        $attributeSet = $this->attributeSetFactory->create();
        $attributeGroupId = $attributeSet->getDefaultGroupId($attributeSetId);


//        if (version_compare($dbVersion, '2.0.1', '<')) {
//            $setup->getConnection()->addColumn(
//                $setup->getTable('sales_order'),
//                'sk_notification_status',
//                [
//                    'type' => \Magento\Framework\DB\Ddl\Table::TYPE_SMALLINT,
//                    'length' => 1,
//                    'nullable' => false,
//                    'default' => 0,
//                    'comment'   => 'Order processed for KpiViews',
//                ]
//            );
//        }
//
//        /** customer_access */
//        if (version_compare($dbVersion, '2.0.2', '<')) {
//            /** @var CustomerSetup $customerSetup */
//            $customerSetup = $this->customerSetupFactory->create(['setup' => $setup]);
//            $customerSetup->addAttribute(
//                Customer::ENTITY,
//                'customer_access',
//                [
//                    'user_defined' => true,
//                    'visible' => true,
//                    'type' => 'varchar',
//                    'label' => 'Customer Access',
//                    'input' => 'select',
//                    'source' => 'KPIs\Customer\Model\Config\Source\CustomerAccess',
//                    'required' => false,
//                    'sort_order' => 100,
//                    'system' => 0,
//                    'position' => 100,
//                    'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
//                    'is_used_in_grid' => true,
//                    'is_visible_in_grid' => true,
//                    'is_filterable_in_grid' => true,
//                    'is_searchable_in_grid' => true,
//                    'backend' => '',
//                ]
//            );
//            $customerSetup->getEavConfig()->getAttribute('customer', 'customer_access')
//                ->addData([
//                    'attribute_set_id' => $attributeSetId,
//                    'attribute_group_id' => $attributeGroupId,
//                    'used_in_forms' => ['adminhtml_customer','customer_account_create','customer_account_edit'],
//                ])
//                ->save();
//        }
//
//        /** valid_until */
//        if (version_compare($dbVersion, '2.0.3', '<')) {
//            /** @var CustomerSetup $customerSetup */
//            $customerSetup = $this->customerSetupFactory->create(['setup' => $setup]);
//            $customerSetup->addAttribute(
//                Customer::ENTITY,
//                'valid_until',
//                [
//                    'user_defined' => true,
//                    'visible' => true,
//                    'type' => 'datetime',
//                    'label' => 'Valid Until',
//                    'input' => 'date',
//                    'source' => '',
//                    'required' => false,
//                    'sort_order' => 100,
//                    'system' => 0,
//                    'input_filter' => 'date',
//                    'validate_rules' => 'a:1:{s:16:"input_validation";s:4:"date";}',
//                    'position' => 122,
//                    'is_used_in_grid' => true,
//                    'is_visible_in_grid' => true,
//                    'is_filterable_in_grid' => true,
//                    'is_searchable_in_grid' => true,
//                    'backend' => 'Magento\Eav\Model\Entity\Attribute\Backend\Datetime',
//                ]
//            );
//            $customerSetup->getEavConfig()->getAttribute('customer', 'valid_until')
//                ->addData([
//                    'attribute_set_id' => $attributeSetId,
//                    'attribute_group_id' => $attributeGroupId,
//                    'used_in_forms' => ['adminhtml_customer','customer_account_create','customer_account_edit'],
//                ])
//                ->save();
//        }
//
//        /** customer_membership */
//        if (version_compare($dbVersion, '2.0.4', '<')) {
//            /** @var CustomerSetup $customerSetup */
//            $customerSetup = $this->customerSetupFactory->create(['setup' => $setup]);
//            $customerSetup->addAttribute(
//                Customer::ENTITY,
//                'customer_membership',
//                [
//                    'user_defined' => true,
//                    'visible' => true,
//                    'type' => 'varchar',
//                    'label' => 'Membership',
//                    'input' => 'select',
//                    'source' => 'KPIs\Customer\Model\Config\Source\Membership',
//                    'required' => false,
//                    'sort_order' => 100,
//                    'system' => 0,
//                    'position' => 121,
//                    'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
//                    'is_used_in_grid' => true,
//                    'is_visible_in_grid' => true,
//                    'is_filterable_in_grid' => true,
//                    'is_searchable_in_grid' => true,
//                    'backend' => '',
//                ]
//            );
//            $customerSetup->getEavConfig()->getAttribute('customer', 'customer_membership')
//                ->addData([
//                    'attribute_set_id' => $attributeSetId,
//                    'attribute_group_id' => $attributeGroupId,
//                    'used_in_forms' => ['adminhtml_customer','customer_account_create','customer_account_edit'],
//                ])
//                ->save();
//
//        }
//
//        /** kpi_views_customer */
//        if (version_compare($dbVersion, '2.0.5', '<')) {
//            /** @var CustomerSetup $customerSetup */
//            $customerSetup = $this->customerSetupFactory->create(['setup' => $setup]);
//            $customerSetup->addAttribute(
//                Customer::ENTITY,
//                'kpi_views_customer',
//                [
//                    'user_defined' => true,
//                    'visible' => true,
//                    'type' => 'varchar',
//                    'label' => 'Kpi Views',
//                    'input' => 'text',
//                    'source' => '',
//                    'required' => false,
//                    'sort_order' => 100,
//                    'system' => 0,
//                    'position' => 100,
//                    'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
//                    'is_used_in_grid' => true,
//                    'is_visible_in_grid' => true,
//                    'is_filterable_in_grid' => true,
//                    'is_searchable_in_grid' => true,
//                    'backend' => '',
//                ]
//            );
//            $customerSetup->getEavConfig()->getAttribute('customer', 'kpi_views_customer')
//                ->addData([
//                    'attribute_set_id' => $attributeSetId,
//                    'attribute_group_id' => $attributeGroupId,
//                    'used_in_forms' => ['adminhtml_customer','customer_account_create','customer_account_edit'],
//                ])
//                ->save();
//        }
    }
}