<?php
/**
 * Copyright © 2016 x-mage2(Yosto). All rights reserved.
 * See README.md for details.
 */
namespace Yosto\CustomerAttribute\Controller\Adminhtml;

use Magento\Backend\App\Action;
use Magento\Framework\Registry;
use Magento\Framework\View\Result\PageFactory;
use Psr\Log\LoggerInterface;
use Yosto\CustomerAttribute\Model\ResourceModel\EavAttribute\CollectionFactory;
use Magento\Ui\Component\MassAction\Filter;
use Yosto\CustomerAttribute\Model\EavAttributeFactory;
use Magento\Customer\Setup\CustomerSetupFactory;

/**
 * Class Attribute
 * @package Yosto\CustomerAttribute\Controller\Adminhtml
 */
abstract class Attribute extends Action
{

    /**
     * @var LoggerInterface
     */
    protected $_logger;

    /**
     * @var PageFactory
     */
    protected $_resultPageFactory;

    /**
     * @var CollectionFactory
     */
    protected $_collectionFactory;

    protected $_entityTypeId;

    protected $_filter;

    protected $_eavAttributeFactory;

    protected $_coreRegistry;

    /**
     * @param Action\Context $context
     * @param LoggerInterface $logger
     * @param PageFactory $pageFactory
     * @param CollectionFactory $collectionFactory
     * @param EavAttributeFactory $eavAttributeFactory
     * @param Filter $filter
     * @param Registry $coreRegistry
     */
    public function __construct(
        Action\Context $context,
        LoggerInterface $logger,
        PageFactory $pageFactory,
        CollectionFactory $collectionFactory,
        EavAttributeFactory $eavAttributeFactory,
        Filter $filter,
        Registry $coreRegistry
    ) {
        parent::__construct($context);
        $this->_logger = $logger;
        $this->_resultPageFactory = $pageFactory;
        $this->_collectionFactory = $collectionFactory;
        $this->_filter = $filter;
        $this->_eavAttributeFactory = $eavAttributeFactory;
        $this->_coreRegistry = $coreRegistry;
    }



    /**
     * Generate code from label
     *
     * @param string $label
     * @return string
     */
    protected function generateCode($label)
    {
        $code = substr(
            preg_replace(
                '/[^a-z_0-9]/',
                '_',
                $this->_objectManager->create('Magento\Catalog\Model\Product\Url')->formatUrlKey($label)
            ),
            0,
            30
        );
        $validatorAttrCode = new \Zend_Validate_Regex(['pattern' => '/^[a-z][a-z_0-9]{0,29}[a-z0-9]$/']);
        if (!$validatorAttrCode->isValid($code)) {
            $code = 'attr_' . ($code ?: substr(md5(time()), 0, 8));
        }
        return $code;
    }


    /**
     * Returns result of authorisation permission
     *
     * @return bool
     */
    protected function _isAllowed()
    {
        return $this->_authorization
            ->isAllowed('Yosto_CustomerAttribute::view_attributes');
    }
    /**
     * Dispatch request
     *
     * @param \Magento\Framework\App\RequestInterface $request
     * @return \Magento\Framework\App\ResponseInterface
     */
    public function dispatch(\Magento\Framework\App\RequestInterface $request)
    {
        $this->_entityTypeId = $this->_objectManager->create(
            'Magento\Eav\Model\Entity'
        )->setType(
            \Magento\Customer\Model\Customer::ENTITY
        )->getTypeId();
        return parent::dispatch($request);
    }


    /**
     * @param \Magento\Framework\Phrase|null $title
     * @return \Magento\Backend\Model\View\Result\Page
     */
    protected function createActionPage($title = null)
    {
        /** @var \Magento\Backend\Model\View\Result\Page $resultPage */
        $resultPage = $this->_resultPageFactory->create();
        if ($this->getRequest()->getParam('popup')) {
            if ($this->getRequest()->getParam('product_tab') == 'variations') {
                $resultPage->addHandle(['popup', 'yosto_customer_attribute_attribute_edit_attribute_tab_variations_popup']);
            } else {
                $resultPage->addHandle(['popup', 'yosto_customer_attribute_attribute_edit_popup']);
            }
            $pageConfig = $resultPage->getConfig();
            $pageConfig->addBodyClass('attribute-popup');
        } else {
            $resultPage->addBreadcrumb(__('Customer'), __('Customer'))
                ->addBreadcrumb(__('Manage Customer Attributes'), __('Manage Customer Attributes'))
                ->setActiveMenu('Yosto_CustomerAttribute::manage_attribute');
            if (!empty($title)) {
                $resultPage->addBreadcrumb($title, $title);
            }
        }
        $resultPage->getConfig()->getTitle()->prepend(__('Customer Attributes'));
        return $resultPage;
    }
}