<?php
/**
 * Copyright  2016 x-mage2(Yosto). All rights reserved.
 * See README.md for details.
 */
namespace Yosto\CustomerAttribute\Controller\Adminhtml\Order;

use Magento\Customer\Api\AddressRepositoryInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Sales\Api\OrderAddressRepositoryInterface;
use Psr\Log\LoggerInterface;

/**
 * Class AddressSavePlugin
 * @package Yosto\CustomerAttribute\Controller\Adminhtml\Order
 */
class AddressSavePlugin
{
    /**
     * @var AddressRepositoryInterface
     */
    protected $_addressRepository;

    /**
     * @var LoggerInterface
     */
    protected $_logger;

    /**
     * @var OrderAddressRepositoryInterface
     */
    protected $_orderAddressRepository;

    /**
     * @param AddressRepositoryInterface $addressRepository
     * @param OrderAddressRepositoryInterface $orderAddressRepository
     * @param LoggerInterface $logger
     */
    public function __construct(
        AddressRepositoryInterface $addressRepository,
        OrderAddressRepositoryInterface $orderAddressRepository,
        LoggerInterface $logger
    ) {
        $this->_addressRepository = $addressRepository;
        $this->_orderAddressRepository = $orderAddressRepository;
        $this->_logger = $logger;
    }

    /**
     * @param \Magento\Sales\Controller\Adminhtml\Order\AddressSave $subject
     */
    public function beforeExecute(\Magento\Sales\Controller\Adminhtml\Order\AddressSave $subject)
    {
        $addressId = $subject->getRequest()->getParam('address_id');
        /** @var $address \Magento\Sales\Api\Data\OrderAddressInterface|\Magento\Sales\Model\Order\Address */
        $address = $this->_orderAddressRepository->get($addressId);
        $data = $subject->getRequest()->getPostValue();
        if ($data && $address->getId()) {
            try {
                if ($address->getCustomerAddressId() != null) {

                    $customerAddress = $this->_addressRepository->getById($address->getCustomerAddressId());
                    foreach ($data as $key=>$value) {
                        $customerAddress->setCustomAttribute($key, $value);
                    }
                    $this->_addressRepository->save($customerAddress);
                }
            } catch (\Exception $e) {
                $this->_logger->error('Admin order update address: ' . $e->getTraceAsString());
            }
        }
    }

}