<?php

namespace KPIs\Customer\Block;

use KPIs\Customer\Helper\Data;
use Magento\Framework\Stdlib\Cookie\CookieMetadataFactory;
use Magento\Framework\Stdlib\CookieManagerInterface;
use Magento\Framework\Session\SessionManagerInterface;
use KPIs\Customer\Logger\Logger;

class SyncLogin  extends \Magento\Framework\View\Element\Template
{
    const COOKIE_NAME_CUSTOMER = 'smartkpis_customer';
    const COOKIE_NAME_CUSTOMER_FLAG = 'smartkpis_customer_flag';

    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $customerSession;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var CookieMetadataFactory
     */
    private $cookieMetadataFactory;

    /**
     * @var CookieManagerInterface
     */
    private $cookieManager;

    /**
     * @var SessionManagerInterface
     */
    private $sessionManagerInterface;

    /**
     * @var Logger
     */
    private $logger;

    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Customer\Model\Session $customerSession,
        CookieMetadataFactory $cookieMetadataFactory,
        CookieManagerInterface $cookieManager,
        SessionManagerInterface $sessionManagerInterface,
        Logger $logger,
        Data $helper,
        array $data = []
    )
    {
        $this->customerSession = $customerSession;
        $this->helper = $helper;
        $this->cookieMetadataFactory = $cookieMetadataFactory;
        $this->cookieManager = $cookieManager;
        $this->sessionManagerInterface = $sessionManagerInterface;
        $this->logger = $logger;
        parent::__construct($context, $data);
    }

    public function getCustomerForSmartkpis()
    {
        $customerForSmartkpis = $this->helper->getCustomerForSmartkpis($this->customerSession);
        if (sizeof($customerForSmartkpis) && empty($this->cookieManager->getCookie(self::COOKIE_NAME_CUSTOMER_FLAG))) {
            $cookieMetadata = $this->cookieMetadataFactory->createPublicCookieMetadata()
                ->setHttpOnly(false)
                ->setDomain($this->sessionManagerInterface->getCookieDomain())
                ->setDuration($this->sessionManagerInterface->getCookieLifetime())
                ->setPath($this->sessionManagerInterface->getCookiePath());

            $this->cookieManager->setPublicCookie(self::COOKIE_NAME_CUSTOMER, json_encode($customerForSmartkpis), $cookieMetadata);
            $this->cookieManager->setPublicCookie(self::COOKIE_NAME_CUSTOMER_FLAG, 1, $cookieMetadata);
        }
        return $customerForSmartkpis;
    }

    public function getSmartkpisLoginUrl()
    {
        return $this->helper->getSmartkpisLoginUrl();
    }

    public function getSmartkpisLogoutUrl()
    {
        return $this->helper->getSmartkpisLogoutUrl();
    }

    public function getCookieFlag()
    {
        return self::COOKIE_NAME_CUSTOMER_FLAG;
    }
    public function getCookieCustomer()
    {
        return self::COOKIE_NAME_CUSTOMER;
    }
}