<?php

namespace KPIs\Customer\Controller\Account;

use Magento\Customer\Model\Session;
use Magento\Framework\App\Action\Context;
use Magento\Framework\View\Result\PageFactory;
use Magento\Customer\Api\AccountManagementInterface;
use KPIs\Customer\Helper\Data;

class Login extends \Magento\Customer\Controller\Account\Login
{
    /**
     * @var Session
     */
    protected $session;

    /**
     * @var PageFactory
     */
    protected $resultPageFactory;

    /** @var AccountManagementInterface */
    protected $customerAccountManagement;


    /**
     * @var Data
     */
    protected $helper;

    /**
     * Login constructor.
     * @param Context $context
     * @param Session $customerSession
     * @param PageFactory $resultPageFactory
     * @param AccountManagementInterface $customerAccountManagement
     * @param Data $helper
     */
    public function __construct(
        Context $context,
        Session $customerSession,
        PageFactory $resultPageFactory,
        AccountManagementInterface $customerAccountManagement,
        Data $helper
    ) {
        $this->session = $customerSession;
        $this->resultPageFactory = $resultPageFactory;
        $this->customerAccountManagement = $customerAccountManagement;
        $this->helper = $helper;
        parent::__construct($context, $customerSession, $resultPageFactory);
    }

    /**
     * Customer login form page
     *
     * @return \Magento\Framework\Controller\Result\Redirect|\Magento\Framework\View\Result\Page
     */
    public function execute()
    {
        $externalLogin = $this->getRequest()->getParam('external_login');

        if (empty($externalLogin)) {
            return parent::execute();
        }

        // $externalLogin = value sent by smartkpiinstitute when ajaxLogin.
        if ($externalLogin && $externalLogin = 1) {
            if ($this->session->isLoggedIn()) {
                $magentoUser = $this->helper->getCustomerForSmartkpis($this->session);
                return $this->_jsonPSuccessExit($magentoUser);
            }

            $username = $this->getRequest()->getParam('username');
            $password = $this->getRequest()->getParam('password');
            if (!empty($username) && !empty($password)) {
                try {
                    $customer = $this->customerAccountManagement->authenticate($username, $password);
                    $this->session->setCustomerDataAsLoggedIn($customer);
                    $this->session->regenerateId();
                    $magentoUser = $this->helper->getCustomerForSmartkpis($this->session);
                    return $this->_jsonPSuccessExit($magentoUser);
                } catch (EmailNotConfirmedException $e) {
                    return $this->_jsonPFailExit(array("error" => __('This account is not confirmed.')));
                } catch (AuthenticationException $e) {
                    return $this->_jsonPFailExit(array("error" => __('Invalid login or password.')));
                } catch (\Exception $e) {
                    return $this->_jsonPFailExit(array("error" => $e->getMessage()));
                }
            } else {
                return $this->_jsonPFailExit(array('error' => __('A login and a password are required.')));
            }
        }
    }

    /**
     * Callback - When login success send to SmartKpiInstitute.
     *
     * @param array $magentoUser
     */
    protected function _jsonPSuccessExit($magentoUser)
    {
        header('Content-type: application/javascript');
        echo 'loginSuccessCallback(' . json_encode($magentoUser) . ')';
        exit();
    }

    /**
     * Callback - When login failed send to SmartKpiInstitute.
     */
    protected function _jsonPFailExit($error)
    {
        header('Content-type: application/javascript');
        echo 'loginFailCallback(' . json_encode($error) . ')';
        exit();
    }
}
