<?php
/**
 * ClassName  SetKpisViews.
 * @developer: Larisa Groza
 * @company: BitStone
 */

namespace KPIs\Customer\Observer;

use Magento\Framework\Stdlib\DateTime;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use KPIs\Customer\Logger\Logger;
use Magento\Framework\Registry;

class CustomerLoadAfter implements ObserverInterface
{
    /**
     * Logger of customer's log data.
     *
     * @var Logger
     */
    protected $logger;

    /**
     * @var \KPIs\Customer\Helper\Data
     */
    protected $helper;

    /**
     * @var Registry
     */
    protected $registry;

    /**
     * CustomerLoadAfter constructor.
     * @param \KPIs\Customer\Helper\Data $helper
     * @param Registry $registry
     * @param Logger $logger
     */
    public function __construct(
        \KPIs\Customer\Helper\Data $helper,
        Registry $registry,
        Logger $logger
    ) {
        $this->helper = $helper;
        $this->logger = $logger;
        $this->registry = $registry;
    }

    /**
     * Register the kpiViews for specific customer.
     *
     * @param Observer $observer
     * @return void
     */
    public function execute(Observer $observer)
    {
        $customer =  $observer->getCustomer();
        $customerId = $customer->getId();
        $kpiViews = $customer->getKpiViewsCustomer();
        $customeraAccess = $customer->getCustomerAccess();

        $this->registry->unregister('kpi_views_customer_' . $customerId);
        $this->registry->register('kpi_views_customer_' . $customerId, $kpiViews);

        $this->registry->unregister('customer_access_' . $customerId);
        $this->registry->register('customer_access_' . $customerId, $customeraAccess);
    }
}