<?php
/**
 * ClassName  SetKpisViews.
 * @developer: Larisa Groza
 * @company: BitStone
 */

namespace KPIs\Customer\Observer;

use Magento\Framework\Stdlib\DateTime;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use KPIs\Customer\Logger\Logger;

class CustomerLogout implements ObserverInterface
{
    /**
     * Logger of customer's log data.
     *
     * @var Logger
     */
    protected $logger;

    /**
     * @var \KPIs\Customer\Helper\Data
     */
    protected $helper;

    /**
     * CustomerLogout constructor.
     * @param \KPIs\Customer\Helper\Data $helper
     * @param Logger $logger
     */
    public function __construct(
        \KPIs\Customer\Helper\Data $helper,
        Logger $logger
    ) {
        $this->helper = $helper;
        $this->logger = $logger;
    }

    /**
     * Handler for 'kpis_views_sync_customer_logout' event triggered when log out
     *
     * @param Observer $observer
     * @return void
     */
    public function execute(Observer $observer)
    {
        $customerId = $observer->getCustomerId();
        $kpiViews = $observer->getKpiViewsCustomer();
        if ($customerId) {
            $this->helper->updateKpiViewsAndAccess($customerId, ['kpi_views_customer' => $kpiViews], false);
        }
    }
}