<?php

namespace KPIs\CustomerAttributes\Setup;

use Magento\Customer\Model\Customer;
use Magento\Customer\Setup\CustomerSetup;
use Magento\Customer\Setup\CustomerSetupFactory;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\UpgradeDataInterface;
use Magento\Eav\Model\Entity\Attribute\Set as AttributeSet;
use Magento\Eav\Model\Entity\Attribute\SetFactory as AttributeSetFactory;

/**
 * Customer Attributes - Upgrade Data.
 *
 * @category   Magento2
 * @author     Larisa Groza
 */
class UpgradeData implements UpgradeDataInterface
{

    /**
     * @var CustomerSetupFactory
     */
    private $customerSetupFactory;
    
    /**
     * @var AttributeSetFactory
     */
    private $attributeSetFactory;

    public function __construct(CustomerSetupFactory $customerSetupFactory, AttributeSetFactory $attributeSetFactory)
    {
        $this->customerSetupFactory = $customerSetupFactory;
        $this->attributeSetFactory = $attributeSetFactory;
    }

    /**
     * Upgrades data for a module
     *
     * @param ModuleDataSetupInterface $setup
     * @param ModuleContextInterface $context
     * @return void
     */
    public function upgrade(ModuleDataSetupInterface $setup, ModuleContextInterface $context)
    {
        $dbVersion = $context->getVersion();

        $customerSetup = $this->customerSetupFactory->create(['setup' => $setup]);

        $customerEntity = $customerSetup->getEavConfig()->getEntityType('customer');
        $attributeSetId = $customerEntity->getDefaultAttributeSetId();

        /** @var $attributeSet AttributeSet */
        $attributeSet = $this->attributeSetFactory->create();
        $attributeGroupId = $attributeSet->getDefaultGroupId($attributeSetId);


        /** functional_area */
        //if (version_compare($dbVersion, '2.0.1', '<')) {
            /** @var CustomerSetup $customerSetup */
            $customerSetup = $this->customerSetupFactory->create(['setup' => $setup]);
            $customerSetup->addAttribute(
                Customer::ENTITY,
                'functional_area',
                [
                    'user_defined' => true,
                    'visible' => true,
                    'type' => 'varchar',
                    'label' => 'Functional Area',
                    'input' => 'select',
                    'source' => 'KPIs\CustomerAttributes\Model\Config\Source\FunctionalArea',
                    'required' => false,
                    'sort_order' => 100,
                    'system' => 0,
                    'position' => 117,
                    'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
                    'is_used_in_grid' => false,
                    'is_visible_in_grid' => false,
                    'is_filterable_in_grid' => false,
                    'is_searchable_in_grid' => false,
                    'backend' => '',
                ]
            );
            $customerSetup->getEavConfig()->getAttribute('customer', 'functional_area')
                ->addData([
                    'attribute_set_id' => $attributeSetId,
                    'attribute_group_id' => $attributeGroupId,
                    'used_in_forms' => ['adminhtml_customer','customer_account_create','customer_account_edit'],
                ])
                ->save();
        // }

        /** industry */
        // if (version_compare($dbVersion, '2.0.2', '<')) {
            /** @var CustomerSetup $customerSetup */
            $customerSetup = $this->customerSetupFactory->create(['setup' => $setup]);
            $customerSetup->addAttribute(
                Customer::ENTITY,
                'industry',
                [
                    'user_defined' => true,
                    'visible' => true,
                    'type' => 'varchar',
                    'label' => 'Industry',
                    'input' => 'select',
                    'source' => 'KPIs\CustomerAttributes\Model\Config\Source\Industry',
                    'required' => true,
                    'sort_order' => 100,
                    'system' => 0,
                    'position' => 100,
                    'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
                    'is_used_in_grid' => false,
                    'is_visible_in_grid' => false,
                    'is_filterable_in_grid' => false,
                    'is_searchable_in_grid' => false,
                    'backend' => '',
                ]
            );
            $customerSetup->getEavConfig()->getAttribute('customer', 'industry')
                ->addData([
                    'attribute_set_id' => $attributeSetId,
                    'attribute_group_id' => $attributeGroupId,
                    'used_in_forms' => ['adminhtml_customer','customer_account_create','customer_account_edit'],
                ])
                ->save();
        // }

        /** job_title */
        // if (version_compare($dbVersion, '2.0.3', '<')) {
            /** @var CustomerSetup $customerSetup */
            $customerSetup = $this->customerSetupFactory->create(['setup' => $setup]);
            $customerSetup->addAttribute(
                Customer::ENTITY,
                'job_title',
                [
                    'user_defined' => true,
                    'visible' => true,
                    'type' => 'varchar',
                    'label' => 'Job Title',
                    'input' => 'text',
                    'source' => '',
                    'required' => true,
                    'sort_order' => 100,
                    'system' => 0,
                    'position' => 118,
                    'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
                    'is_used_in_grid' => false,
                    'is_visible_in_grid' => false,
                    'is_filterable_in_grid' => false,
                    'is_searchable_in_grid' => false,
                    'backend' => '',
                ]
            );
            $customerSetup->getEavConfig()->getAttribute('customer', 'job_title')
                ->addData([
                    'attribute_set_id' => $attributeSetId,
                    'attribute_group_id' => $attributeGroupId,
                    'used_in_forms' => ['adminhtml_customer','customer_account_create','customer_account_edit'],
                ])
                ->save();
        // }

        /** organization */
        //if (version_compare($dbVersion, '2.0.4', '<')) {
            /** @var CustomerSetup $customerSetup */
            $customerSetup = $this->customerSetupFactory->create(['setup' => $setup]);
            $customerSetup->addAttribute(
                Customer::ENTITY,
                'organization',
                [
                    'user_defined' => true,
                    'visible' => true,
                    'type' => 'varchar',
                    'label' => 'Organization',
                    'input' => 'text',
                    'source' => '',
                    'required' => true,
                    'sort_order' => 100,
                    'system' => 0,
                    'position' => 100,
                    'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
                    'is_used_in_grid' => false,
                    'is_visible_in_grid' => false,
                    'is_filterable_in_grid' => false,
                    'is_searchable_in_grid' => false,
                    'backend' => '',
                ]
            );
            $customerSetup->getEavConfig()->getAttribute('customer', 'organization')
                ->addData([
                    'attribute_set_id' => $attributeSetId,
                    'attribute_group_id' => $attributeGroupId,
                    'used_in_forms' => ['adminhtml_customer','customer_account_create','customer_account_edit'],
                ])
                ->save();
        //}

        /** organization_size */
        //if (version_compare($dbVersion, '2.0.5', '<')) {
            /** @var CustomerSetup $customerSetup */
            $customerSetup = $this->customerSetupFactory->create(['setup' => $setup]);
            $customerSetup->addAttribute(
                Customer::ENTITY,
                'organization_size',
                [
                    'user_defined' => true,
                    'visible' => true,
                    'type' => 'varchar',
                    'label' => 'Organization Size',
                    'input' => 'select',
                    'source' => 'KPIs\CustomerAttributes\Model\Config\Source\OrganizationSize',
                    'required' => false,
                    'sort_order' => 116,
                    'system' => 0,
                    'position' => 116,
                    'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
                    'is_used_in_grid' => false,
                    'is_visible_in_grid' => false,
                    'is_filterable_in_grid' => false,
                    'is_searchable_in_grid' => false,
                    'backend' => '',
                ]
            );
            $customerSetup->getEavConfig()->getAttribute('customer', 'organization_size')
                ->addData([
                    'attribute_set_id' => $attributeSetId,
                    'attribute_group_id' => $attributeGroupId,
                    'used_in_forms' => ['adminhtml_customer','customer_account_create','customer_account_edit'],
                ])
                ->save();
        //}

        /** title */
        //if (version_compare($dbVersion, '2.0.6', '<')) {
            /** @var CustomerSetup $customerSetup */
            $customerSetup = $this->customerSetupFactory->create(['setup' => $setup]);
            $customerSetup->addAttribute(
                Customer::ENTITY,
                'title',
                [
                    'user_defined' => true,
                    'visible' => true,
                    'type' => 'varchar',
                    'label' => 'Title',
                    'input' => 'select',
                    'source' => 'KPIs\CustomerAttributes\Model\Config\Source\Title',
                    'required' => true,
                    'sort_order' => 100,
                    'system' => 0,
                    'position' => 119,
                    'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_GLOBAL,
                    'is_used_in_grid' => false,
                    'is_visible_in_grid' => false,
                    'is_filterable_in_grid' => false,
                    'is_searchable_in_grid' => false,
                    'backend' => '',
                ]
            );
            $customerSetup->getEavConfig()->getAttribute('customer', 'title')
                ->addData([
                    'attribute_set_id' => $attributeSetId,
                    'attribute_group_id' => $attributeGroupId,
                    'used_in_forms' => ['adminhtml_customer','customer_account_create','customer_account_edit'],
                ])
                ->save();
        // }
    }
}