<?php
/**
 * Copyright © 2016 x-mage2(Yosto). All rights reserved.
 * See README.md for details.
 */
namespace Yosto\CustomerAttribute\Block\Adminhtml\Attribute\Edit\Tab;

use Magento\Backend\Block\Widget\Form\Generic;
use Magento\Backend\Block\Template\Context;
use Magento\Backend\Block\Widget\Form;
use Magento\Config\Model\Config\Source\Yesno;
use Yosto\CustomerAttribute\Model\FormFactory as CustomerFormAttributeFactory;
use Yosto\CustomerAttribute\Block\Adminhtml\Attribute\PropertyLocker;
use Magento\Framework\Data\FormFactory;
use Magento\Framework\Registry;

/**
 * Config visible in forms for customer/address attribute
 *
 * Class Front
 * @package Yosto\CustomerAttribute\Block\Adminhtml\Attribute\Edit\Tab
 */
class Front extends Generic
{
    /**
     * @var Yesno
     */
    protected $_yesNo;

    /**
     * @var PropertyLocker
     */
    private $propertyLocker;

    /**
     * @var CustomerFormAttributeFactory
     */
    protected $_customerFormAttributeFactory;

    /**
     * @param Context $context
     * @param Registry $registry
     * @param FormFactory $formFactory
     * @param Yesno $yesNo
     * @param PropertyLocker $propertyLocker
     * @param CustomerFormAttributeFactory $customerFormAttributeFactory
     * @param array $data
     */
    public function __construct(
        Context $context,
        Registry $registry,
        FormFactory $formFactory,
        Yesno $yesNo,
        PropertyLocker $propertyLocker,
        CustomerFormAttributeFactory $customerFormAttributeFactory,
        array $data = []
    )
    {
        $this->_yesNo = $yesNo;
        $this->propertyLocker = $propertyLocker;
        $this->_customerFormAttributeFactory = $customerFormAttributeFactory;
        parent::__construct($context, $registry, $formFactory, $data);
    }

    /**
     * @return $this
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    protected function _prepareForm()
    {
        /** @var Attribute $attributeObject */
        $attributeObject = $this->_coreRegistry->registry('customer_entity_attribute');

        /** @var \Magento\Framework\Data\Form $form */
        $form = $this->_formFactory->create(
            ['data' => ['id' => 'edit_form', 'action' => $this->getData('action'), 'method' => 'post']]
        );

        $isUsedInAdminhtmlCustomer = 0;
        $isUsedInAdminhtmlCheckout = 0;
        $isUsedInAdminhtmlCustomerAddress = 0;
        $isUsedInCustomerAccountCreate = 0;
        $isUsedInCustomerAccountEdit = 0;
        $isUsedInCustomerRegisterAddress = 0;
        $isUsedInCustomerAddressEdit = 0;
        $formModel = $this->_customerFormAttributeFactory->create();
        if ($attributeObject) {
            $usedInForms = $formModel->getCollection()->distinct(false)
                ->addFieldToFilter(
                    'attribute_id',
                    $attributeObject->getData('attribute_id')
                );
            foreach ($usedInForms as $item) {
                if ($item['form_code'] == 'adminhtml_customer') {

                    $isUsedInAdminhtmlCustomer = 1;

                } elseif ($item['form_code'] == 'adminhtml_checkout') {

                    $isUsedInAdminhtmlCheckout = 1;

                } elseif ($item['form_code'] == 'adminhtml_customer_address') {

                    $isUsedInAdminhtmlCustomerAddress = 1;

                } elseif ($item['form_code'] == 'customer_account_create') {

                    $isUsedInCustomerAccountCreate = 1;

                } elseif ($item['form_code'] == 'customer_account_edit') {

                    $isUsedInCustomerAccountEdit = 1;

                } elseif ($item['form_code'] == 'customer_register_address') {

                    $isUsedInCustomerRegisterAddress = 1;

                } elseif ($item['form_code'] == 'customer_address_edit') {

                    $isUsedInCustomerAddressEdit = 1;

                }
            }
            /** Set values for data */
            $attributeObject->setData('adminhtml_customer', $isUsedInAdminhtmlCustomer);
            $attributeObject->setData('adminhtml_checkout', $isUsedInAdminhtmlCheckout);
            $attributeObject->setData('adminhtml_customer_address', $isUsedInAdminhtmlCustomerAddress);
            $attributeObject->setData('customer_account_create', $isUsedInCustomerAccountCreate);
            $attributeObject->setData('customer_account_edit', $isUsedInCustomerAccountEdit);
            $attributeObject->setData('customer_register_address', $isUsedInCustomerRegisterAddress);
            $attributeObject->setData('customer_address_edit', $isUsedInCustomerAddressEdit);

        }

        $yesno = $this->_yesNo->toOptionArray();

        $fieldset = $form->addFieldset(
            'front_fieldset',
            ['legend' => __('Storefront Properties'), 'collapsable' => $this->getRequest()->has('popup')]
        );
        $fieldset->addField(
            'entity_type_id',
            'hidden',
            [
                'name' => 'entity_type_id',
                'value' => $this->getRequest()->getParam('entity_type_id')
            ]
        );

        $fieldset->addField(
            'is_visible',
            'select',
            [
                'name' => 'is_visible',
                'label' => __('Visible'),
                'values' => $yesno,
                'note' => "Select 'yes' to show attribute on the admin page and the frontend page"
            ]
        );

        $fieldset->addField(
            'adminhtml_customer',
            'select',
            [
                'name' => 'adminhtml_customer',
                'label' => __('Show on the admin customer page'),
                'values' => $yesno,
            ]
        );

        $fieldset->addField(
            'adminhtml_checkout',
            'select',
            [
                'name' => 'adminhtml_checkout',
                'label' => __('Show on the admin checkout page'),
                'values' => $yesno,
            ]
        );

        $fieldset->addField(
            'customer_account_create',
            'select',
            [
                'name' => 'customer_account_create',
                'label' => __('Show on the registration page'),
                'values' => $yesno,
            ]
        );
        $fieldset->addField(
            'customer_account_edit',
            'select',
            [
                'name' => 'customer_account_edit',
                'label' => __('Show on the Account edit page'),
                'values' => $yesno,
            ]
        );
        $fieldset->addField(
            'adminhtml_customer_address',
            'select',
            [
                'name' => 'adminhtml_customer_address',
                'label' => __('Show on the admin customer address tab'),
                'values' => $yesno,
            ]
        );


        $fieldset->addField(
            'customer_register_address',
            'select',
            [
                'name' => 'customer_register_address',
                'label' => __('Show on the customer address registration form'),
                'values' => $yesno,
                'note' => __('Also visible on the billing/shipping address form (in checkout process)')
            ]
        );

        $fieldset->addField(
            'customer_address_edit',
            'select',
            [
                'name' => 'customer_address_edit',
                'label' => __('Show on the customer address edit page'),
                'values' => $yesno,
            ]
        );

        $fieldset->addField(
            'sort_order',
            'text',
            [
                'name' => 'sort_order',
                'label' => __('Sorting Order'),
                'note' => __('The order to display field on frontend'),
            ]
        );

        $this->setForm($form);
        $form->setValues($attributeObject->getData());
        $this->propertyLocker->lock($form);
        return parent::_prepareForm();

    }
}