<?php
/**
 * Copyright  2016 x-mage2(Yosto). All rights reserved.
 * See README.md for details.
 */

namespace Yosto\CustomerAttribute\Block\Adminhtml\Order\View;

use Magento\Customer\Api\AddressRepositoryInterface;
use Magento\Framework\Registry;
use Psr\Log\LoggerInterface;

/**
 * Class InfoPlugin
 * @package Yosto\CustomerAttribute\Block\Adminhtml\Order\View
 */
class InfoPlugin
{
    /**
     * @var AddressRepositoryInterface
     */
    protected $_addressRepository;

    /**
     * @var Registry
     */
    protected $_coreRegistry;

    /**
     * @var LoggerInterface
     */
    protected $_logger;

    /**
     * @param AddressRepositoryInterface $addressRepository
     * @param Registry $coreRegistry
     * @param LoggerInterface $logger
     */
    public function __construct(
        AddressRepositoryInterface $addressRepository,
        Registry $coreRegistry,
        LoggerInterface $logger
    ) {
        $this->_addressRepository = $addressRepository;
        $this->_coreRegistry = $coreRegistry;
        $this->_logger = $logger;
    }

    /**
     * @param \Magento\Sales\Block\Adminhtml\Order\View\Info $subject
     * @param \Magento\Sales\Model\Order\Address $address
     */
    public function beforeGetFormattedAddress(
        \Magento\Sales\Block\Adminhtml\Order\View\Info $subject,
        \Magento\Sales\Model\Order\Address $address
    ) {
        /** @var \Magento\Sales\Model\Order\Address $address */
        if($address->getCustomerAddressId() != null) {
            try {
                $customerAddress = $this->_addressRepository->getById($address->getCustomerAddressId());
                $customAttributes = $customerAddress->getCustomAttributes();
                foreach ($customAttributes as $attribute) {
                    $address->setData(
                        $attribute->getAttributeCode(),
                        $attribute->getValue()
                    );
                }
            } catch (\Exception $e) {
                $this->_logger->error('Admin Order View Info Plugin: ' . $e->getTraceAsString());
            }
        }

    }
}