<?php
/**
 * Copyright © 2016 x-mage2(Yosto). All rights reserved.
 * See README.md for details.
 */
namespace Yosto\CustomerAttribute\Controller\Adminhtml\Attribute;

use Magento\Customer\Setup\CustomerSetupFactory;
use Magento\Eav\Model\AttributeRepository;
use Magento\Framework\Controller\Result;
use Magento\Framework\View\Result\PageFactory;
use Yosto\CustomerAttribute\Controller\Adminhtml\Attribute;
use Yosto\CustomerAttribute\Model\EavAttributeFactory;
use Yosto\CustomerAttribute\Model\ResourceModel\EavAttribute\CollectionFactory;
use Magento\Ui\Component\MassAction\Filter;
use Magento\Eav\Model\Entity\Attribute\Set as AttributeSet;
use Magento\Eav\Model\Entity\Attribute\SetFactory as AttributeSetFactory;

/**
 * Save customera and address attribute
 *
 * Class Save
 * @package Yosto\CustomerAttribute\Controller\Adminhtml\Attribute
 */
class Save extends Attribute
{
    /**
     * @var
     */
    protected $_attributeLabelCache;

    /**
     * @var \Magento\Catalog\Model\Product\AttributeSet\BuildFactory
     */
    protected $buildFactory;

    /**
     * @var \Magento\Framework\Filter\FilterManager
     */
    protected $filterManager;

    /**
     * @var \Magento\Catalog\Helper\Product
     */
    protected $productHelper;

    /**
     * @var \Magento\Catalog\Model\ResourceModel\Eav\AttributeFactory
     */
    protected $attributeFactory;

    /**
     * @var \Magento\Eav\Model\Adminhtml\System\Config\Source\Inputtype\ValidatorFactory
     */
    protected $validatorFactory;

    /**
     * @var \Magento\Eav\Model\ResourceModel\Entity\Attribute\Group\CollectionFactory
     */
    protected $groupCollectionFactory;


    /**
     * @var AttributeSetFactory
     */
    protected $_attributeSetFactory;


    /**
     * var AttributeRepository
     */
    protected $_attributeRepository;

    /**
     * @var
     */
    protected $_eavConfig;

    /**
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Magento\Framework\Registry $coreRegistry
     * @param PageFactory $resultPageFactory
     * @param \Magento\Catalog\Model\Product\AttributeSet\BuildFactory $buildFactory
     * @param \Yosto\CustomerAttribute\Model\ResourceModel\Eav\AttributeFactory $attributeFactory
     * @param \Magento\Eav\Model\Adminhtml\System\Config\Source\Inputtype\ValidatorFactory $validatorFactory
     * @param \Magento\Eav\Model\ResourceModel\Entity\Attribute\Group\CollectionFactory $groupCollectionFactory
     * @param \Magento\Framework\Filter\FilterManager $filterManager
     * @param \Magento\Catalog\Helper\Product $productHelper
     * @param EavAttributeFactory $eavAttributeFactory
     * @param \Psr\Log\LoggerInterface $logger
     * @param CollectionFactory $collectionFactory
     * @param Filter $filter
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        \Magento\Framework\Registry $coreRegistry,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Magento\Catalog\Model\Product\AttributeSet\BuildFactory $buildFactory,
        \Yosto\CustomerAttribute\Model\ResourceModel\Eav\AttributeFactory $attributeFactory,
        \Magento\Eav\Model\Adminhtml\System\Config\Source\Inputtype\ValidatorFactory $validatorFactory,
        \Magento\Eav\Model\ResourceModel\Entity\Attribute\Group\CollectionFactory $groupCollectionFactory,
        \Magento\Framework\Filter\FilterManager $filterManager,
        \Magento\Catalog\Helper\Product $productHelper,
        EavAttributeFactory $eavAttributeFactory,
        \Psr\Log\LoggerInterface $logger,
        CollectionFactory $collectionFactory,
        Filter $filter,
        AttributeSetFactory $attributeSetFactory,
        AttributeRepository $attributeRepository,
        \Magento\Eav\Model\ConfigFactory $eavConfig
    )
    {
        parent::__construct(
            $context,
            $logger,
            $resultPageFactory,
            $collectionFactory,
            $eavAttributeFactory,
            $filter,
            $coreRegistry
        );
        $this->buildFactory = $buildFactory;
        $this->filterManager = $filterManager;
        $this->productHelper = $productHelper;
        $this->attributeFactory = $attributeFactory;
        $this->validatorFactory = $validatorFactory;
        $this->groupCollectionFactory = $groupCollectionFactory;
        $this->_attributeSetFactory = $attributeSetFactory;
        $this->_attributeRepository = $attributeRepository;
        $this->_eavConfig = $eavConfig;
    }

    /**
     * @return \Magento\Backend\Model\View\Result\Redirect
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @SuppressWarnings(PHPMD.NPathComplexity)
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     */
    public function execute()
    {
        $data = $this->getRequest()->getPostValue();
        $resultRedirect = $this->resultRedirectFactory->create();
        if ($data) {

            $redirectBack = $this->getRequest()->getParam('back', false);
            /* @var $model \Yosto\CustomerAttribute\Model\ResourceModel\Eav\Attribute */
            $model = $this->attributeFactory->create();

            $attributeId = $this->getRequest()->getParam('attribute_id');

            $attributeCode = $this->getRequest()->getParam('attribute_code');
            $frontendLabel = $this->getRequest()->getParam('frontend_label');
            $attributeCode = $attributeCode ?: $this->generateCode($frontendLabel[0]);
            if (strlen($this->getRequest()->getParam('attribute_code')) > 0) {
                $validatorAttrCode = new \Zend_Validate_Regex(['pattern' => '/^[a-z][a-z_0-9]{0,30}$/']);
                if (!$validatorAttrCode->isValid($attributeCode)) {
                    $this->messageManager->addError(
                        __(
                            'Attribute code "%1" is invalid. Please use only letters (a-z), ' .
                            'numbers (0-9) or underscore(_) in this field, first character should be a letter.',
                            $attributeCode
                        )
                    );
                    return $resultRedirect->setPath('yosto_customer_attribute/*/edit', ['attribute_id' => $attributeId, '_current' => true]);
                }
            }
            $data['attribute_code'] = $attributeCode;

            //validate frontend_input
            if (isset($data['frontend_input'])) {
                /** @var $inputType \Magento\Eav\Model\Adminhtml\System\Config\Source\Inputtype\Validator */
                $inputType = $this->validatorFactory->create();
                if (!$inputType->isValid($data['frontend_input'])) {
                    foreach ($inputType->getMessages() as $message) {
                        $this->messageManager->addError($message);
                    }
                    return $resultRedirect->setPath('yosto_customer_attribute/*/edit', ['attribute_id' => $attributeId, '_current' => true]);
                }
            }
            if ($data['entity_type_id']) {
                $this->_entityTypeId = $data['entity_type_id'];
            }
            if ($attributeId) {
                $model->load($attributeId);
                if (!$model->getId()) {
                    $this->messageManager->addError(__('This attribute no longer exists.'));
                    return $resultRedirect->setPath('yosto_customer_attribute/*/');
                }
                // entity type check
                if ($model->getEntityTypeId() != $this->_entityTypeId) {
                    $this->messageManager->addError(__('We can\'t update the attribute.'));
                    $this->_session->setAttributeData($data);
                    return $resultRedirect->setPath('yosto_customer_attribute/*/');
                }

                $data['attribute_code'] = $model->getAttributeCode();
                $data['is_user_defined'] = $model->getIsUserDefined();
                $data['frontend_input'] = $model->getFrontendInput();
            } else {
                /**
                 * @todo add to helper and specify all relations for properties
                 */
                $data['source_model'] = $this->productHelper->getAttributeSourceModelByInputType(
                    $data['frontend_input']
                );
                $data['backend_model'] = $this->productHelper->getAttributeBackendModelByInputType(
                    $data['frontend_input']
                );
            }


            if (is_null($model->getIsUserDefined()) || $model->getIsUserDefined() != 0) {
                $data['backend_type'] = $model->getBackendTypeByInput($data['frontend_input']);
            }

            $defaultValueField = $model->getDefaultValueByInput($data['frontend_input']);
            if ($defaultValueField) {
                $data['default_value'] = $this->getRequest()->getParam($defaultValueField);
            }

            $model->addData($data);

            if (!$attributeId) {
                $model->setEntityTypeId($this->_entityTypeId);
                $model->setIsUserDefined(1);
            }


            /**
             * Check visible of attribute in each customer form
             */
            $isVisibleInAdminCustomer = array_key_exists('adminhtml_customer', $data) ? $data['adminhtml_customer'] : 0;
            $isVisibleInAdminCheckout = array_key_exists('adminhtml_checkout', $data) ? $data['adminhtml_checkout'] : 0;
            $isVisibleInAdminCustomerAddress = array_key_exists('adminhtml_customer_address', $data) ? $data['adminhtml_customer_address'] : 0;
            $isVisibleInRegistration = array_key_exists('customer_account_create', $data) ? $data['customer_account_create'] : 0;
            $isVisibleInAccountEdit = array_key_exists('customer_account_edit', $data) ? $data['customer_account_edit'] : 0;
            $isVisibleInRegisterAddress = array_key_exists('customer_register_address', $data) ? $data['customer_register_address'] : 0;
            $isVisibleInAddressEdit = array_key_exists('customer_address_edit', $data) ? $data['customer_address_edit'] : 0;


            try {
                $model->save();
                // $customerSetup = $this->_customerSetupFactory->create();

                $my_attribute = $this->_attributeRepository->get($model->getEntityType()->getEntityTypeCode(), $model->getAttributeCode());
                $isCustomerEntity = 0;

                /**
                 * Whether attribute type is customer
                 */
                if ($isVisibleInAdminCheckout || $isVisibleInAdminCustomer || $isVisibleInRegistration || $isVisibleInAccountEdit) {
                    $isCustomerEntity = 1;
                }

                /**
                 * Set forms where attribute shows up.
                 */

                $used_in_forms = [];
                if ($isVisibleInAdminCustomer) {
                    $used_in_forms[] = "adminhtml_customer";
                }

                if ($isVisibleInAdminCheckout) {
                    $used_in_forms[] = "adminhtml_checkout";
                }

                if ($isVisibleInAdminCustomerAddress) {
                    $used_in_forms[] = "adminhtml_customer_address";
                }

                if ($isVisibleInRegistration) {
                    $used_in_forms[] = "customer_account_create";
                }

                if ($isVisibleInAccountEdit) {
                    $used_in_forms[] = "customer_account_edit";
                }

                if ($isVisibleInRegisterAddress) {
                    $used_in_forms[] = "customer_register_address";
                }

                if ($isVisibleInAddressEdit) {
                    $used_in_forms[] = "customer_address_edit";
                }


                /**
                 * If no form is selected, attribute belongs to customer entity.
                 */
                if (count($used_in_forms) == 0) {
                    $isCustomerEntity = 1;
                }


                // Check attribute_set & attribute_group for customer

                $customerEntity = $this->_eavConfig->create()->getEntityType('customer');
                $customerAttributeSetId = $customerEntity->getDefaultAttributeSetId();

                /** @var $attributeSet AttributeSet */
                $customerAttributeSet = $this->_attributeSetFactory->create();
                $customerAttributeGroupId = $customerAttributeSet->getDefaultGroupId($customerAttributeSetId);

                // Check attribute_set & attribute_group for address
                $customerAddressEntity = $this->_eavConfig->create()->getEntityType('customer_address');
                $customerAddressSetId = $customerAddressEntity->getDefaultAttributeSetId();

                /** @var  $customerAddressAttributeSet */
                $customerAddressAttributeSet = $this->_attributeSetFactory->create();
                $customerAddressAttributeGroupId = $customerAddressAttributeSet->getDefaultGroupId($customerAddressSetId);

                /**
                 * Set forms
                 */
                $my_attribute->setData('attribute_id', $model->getAttributeId())
                    ->setData("used_in_forms", $used_in_forms)
                    ->setData("is_used_for_customer_segment", true)
                    ->setData("is_system", 0)
                    ->setData("is_user_defined", 1)
                    ->setData("is_visible", $data['is_visible']);

                if ($isCustomerEntity == 1) {
                    $my_attribute->setData('attribute_set_id', $customerAttributeSetId)
                        ->setData('attribute_group_id', $customerAttributeGroupId);
                } else {
                    $my_attribute->setData('attribute_set_id', $customerAddressSetId)
                        ->setData('attribute_group_id', $customerAddressAttributeGroupId);

                }
                $my_attribute->save();

                $addressAttribute = $this->_eavAttributeFactory->create();
                $addressAttribute->load($my_attribute->getAttributeId());

                /**
                 * Set correct entity type for attribute based on form
                 */
                $addressAttribute->setData(
                    'entity_type_id',
                    $isCustomerEntity == 0
                        ? $customerAddressEntity->getEntityTypeId()
                        : $customerEntity->getEntityTypeId()
                );
                $addressAttribute->save();

                $this->messageManager->addSuccess(__('You saved the customer attribute.'));
                $this->_session->setAttributeData(false);
                if ($this->getRequest()->getParam('popup')) {
                    $requestParams = [
                        'attributeId' => $this->getRequest()->getParam('product'),
                        'attribute' => $model->getId(),
                        '_current' => true,
                        'product_tab' => $this->getRequest()->getParam('product_tab'),
                    ];
                    $resultRedirect->setPath('yosto_customer_attribute/attribute/new', $requestParams);
                } elseif ($redirectBack) {
                    $resultRedirect->setPath('yosto_customer_attribute/*/edit', ['attribute_id' => $model->getId(), '_current' => true]);
                } else {
                    $resultRedirect->setPath('yosto_customer_attribute/*/');
                }
                return $resultRedirect;
            } catch (\Exception $e) {
                $this->messageManager->addError($e->getMessage());
                $this->_session->setAttributeData($data);
                return $resultRedirect->setPath('yosto_customer_attribute/*/edit', ['attribute_id' => $attributeId, '_current' => true]);
            }
        }
        return $resultRedirect->setPath('yosto_customer_attribute/*/');
    }


    /**
     * Returns result of authorisation permission
     *
     * @return bool
     */
    protected function _isAllowed()
    {
        return $this->_authorization
            ->isAllowed('Yosto_CustomerAttribute::new_attribute');
    }
}