<?php
/**
 * Copyright © 2016 x-mage2(Yosto). All rights reserved.
 * See README.md for details.
 */
namespace Yosto\CustomerAttribute\Observer;


use Magento\Customer\Api\AddressRepositoryInterface;
use Magento\Customer\Model\AddressFactory;
use Magento\Framework\App\ProductMetadata;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Sales\Api\OrderAddressRepositoryInterface;
use Magento\Sales\Model\Order\AddressFactory as OrderAddressFactory;
/**
 * Class SaveCustomAttributeToAddress
 * @package Yosto\CustomerAttribute\Observer
 */
class SaveCustomAttributeToAddress implements ObserverInterface
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * @var \Magento\Framework\ObjectManagerInterface
     */
    protected $_objectManager;

    /**
     * @var AddressFactory
     */
    protected $_addressFactory;

    /**
     * @var AddressRepositoryInterface
     */
    protected $_addressRepository;

    /**
     * @var
     */
    protected $_productMetadata;
    /**
     * @var OrderAddressFactory
     */
    protected $_orderAddressFactory;

    /**
     * @var OrderAddressRepositoryInterface
     */
    protected $_orderAddressRepository;

    /**
     * @param \Magento\Framework\ObjectManagerInterface $objectmanager
     * @param AddressFactory $addressFactory
     * @param AddressRepositoryInterface $addressRepository
     * @param \Psr\Log\LoggerInterface $logger
     * @param ProductMetadata $productMetadata
     * @param OrderAddressRepositoryInterface $orderAddressRepository
     * @param OrderAddressFactory $orderAddressFactory
     */
    public function __construct(
        \Magento\Framework\ObjectManagerInterface $objectmanager,
        AddressFactory $addressFactory,
        AddressRepositoryInterface $addressRepository,
        \Psr\Log\LoggerInterface $logger,
        ProductMetadata $productMetadata,
        OrderAddressRepositoryInterface $orderAddressRepository,
        OrderAddressFactory $orderAddressFactory
    ) {
        $this->_addressRepository = $addressRepository;
        $this->_addressFactory = $addressFactory;
        $this->_logger = $logger;
        $this->_productMetadata = $productMetadata;
        $this->_objectManager = $objectmanager;
        $this->_orderAddressRepository = $orderAddressRepository;
        $this->_orderAddressFactory = $orderAddressFactory;
    }

    /**
     * @param $orderId
     * @param $isBillingAddress
     * @param $customerAddressId
     * @param $customerId
     * @param $quoteId
     */

    public function updateOrderAddress(
        $orderId,
        $isBillingAddress,
        $customerAddressId,
        $customerId,
        $quoteId
    ) {

        try{
            $orderBillingAddress = $this->_orderAddressFactory->create()
                ->getCollection()
                ->addFieldToFilter('parent_id', $orderId)
                ->addFieldToFilter('address_type', 'billing')
                ->getFirstItem();

            $orderShippingAddress = $this->_orderAddressFactory->create()
                ->getCollection()
                ->addFieldToFilter('parent_id', $orderId)
                ->addFieldToFilter('address_type', 'shipping')
                ->getFirstItem();
            if ($isBillingAddress) {
                if ($orderBillingAddress != null) {
                    $orderBillingAddress->setData('customer_address_id', $customerAddressId);
                    $orderBillingAddress->setData('customer_id', $customerId);
                    $orderBillingAddress->setData('quote_id', $quoteId);
                    $this->_orderAddressRepository->save($orderBillingAddress);
                }
            } else {
                if ($orderShippingAddress != null) {
                    $orderShippingAddress->setData('customer_address_id', $customerAddressId);
                    $orderShippingAddress->setData('customer_id', $customerId);
                    $orderShippingAddress->setData('quote_id', $quoteId);
                    $this->_orderAddressRepository->save($orderShippingAddress);
                }
            }
        } catch (\Exception $e) {
            $this->_logger->error('Could not update order addresses');
        }
    }


    /**
     * @return bool
     */
    public function checkVersion() {
        $currentVersion = $this->_productMetadata->getVersion();
        if(version_compare($currentVersion, '2.1.2') < 0 ) {
            return false;
        }
        return true;
    }

    /**
     * @param $jsonData
     * @return bool
     */
    public function isNewAddress($jsonData)
    {

        foreach ($jsonData as $node) {
            if ($node->value != null && $node->value != '') {
                return true;
            }
        }
        return false;
    }

    /**
     * @param $entityId
     * @param $customAddressData
     */
    public function updateCustomerAddress($entityId, $customAddressData)
    {
        $customerAddress = $this->_addressFactory->create();

        $address = $this->_addressFactory->create()
            ->getCollection()
            ->addFieldToFilter('entity_id', $entityId)
            ->getFirstItem();

        $customerAddress->load($address->getId());
        try {

            $data = $customAddressData;
            $addressData = $customerAddress->getDataModel();
            foreach ($data as $item) {
                $addressData->setCustomAttribute($item->name, $item->value);
            }
            $customerAddress->updateData($addressData);
            $customerAddress->save();

            $this->_logger->debug('Save success');
        } catch (\Exception $e) {
            $this->_logger->debug($e->getTraceAsString());
        }
    }
    /**
     * @param \Magento\Quote\Model\Quote $quote
     * @param $customShippingAddress
     * @param $customBillingAddress
     * @param $shippingCustomerAddressId
     * @param $billingCustomerAddressId
     * @param $isShippingSaveInAddressBook
     * @param $isBillingSaveInAddressBook
     */
    public function saveCustomAttributesForAddressIfVersionTrue(
        $quote,
        $customShippingAddress,
        $customBillingAddress,
        $shippingCustomerAddressId,
        $billingCustomerAddressId,
        $isShippingSaveInAddressBook,
        $isBillingSaveInAddressBook

    ) {
       // Magento 2.1.2 and later version always set customer address id for a quote address.
        $customShippingAddress = json_decode($customShippingAddress);
        $customBillingAddress = json_decode($customBillingAddress);
        if ($this->isNewAddress($customBillingAddress)) {
            if ($isBillingSaveInAddressBook) {
                $this->updateCustomerAddress($billingCustomerAddressId, $customBillingAddress);
            }
        }
        if ($this->isNewAddress($customShippingAddress)) {
            if ($isShippingSaveInAddressBook) {
                $this->updateCustomerAddress($shippingCustomerAddressId, $customShippingAddress);
            }
        }
    }

    /**
     * @param \Magento\Sales\Api\Data\OrderInterface $order
     * @param $quote
     * @param $customShippingAddress
     * @param $customBillingAddress
     * @param $shippingCustomerAddressId
     * @param $billingCustomerAddressId
     * @param $isShippingSaveInAddressBook
     * @param $isBillingSaveInAddressBook
     */
    public function saveCustomAttributesForAddressIfVersionFalse(
        $order,
        $quote,
        $customShippingAddress,
        $customBillingAddress,
        $shippingCustomerAddressId,
        $billingCustomerAddressId,
        $isShippingSaveInAddressBook,
        $isBillingSaveInAddressBook

    ) {
        if (!$shippingCustomerAddressId && !$billingCustomerAddressId) {
            if ($isShippingSaveInAddressBook && !$isBillingSaveInAddressBook) {
                $customerAddress = $this->_addressFactory->create();

                $address = $this->_addressFactory->create()
                    ->getCollection()
                    ->addFieldToFilter('parent_id', $quote->getCustomerId())
                    ->addOrder('entity_id', 'desc')->getFirstItem();

                $customerAddress->load($address->getId());
                try {

                    $data = json_decode($customShippingAddress);
                    $addressData = $customerAddress->getDataModel();
                    foreach ($data as $item) {
                        $addressData->setCustomAttribute($item->name, $item->value);
                    }
                    $customerAddress->updateData($addressData);
                    $customerAddress->save();
                    $this->updateOrderAddress(
                        $order->getId(),
                        false,
                        $customerAddress->getId(),
                        $order->getCustomerId(),
                        $order->getQuoteId()
                    );
                    $this->_logger->debug('Save success');
                } catch (\Exception $e) {
                    $this->_logger->debug($e->getTraceAsString());
                }
            } elseif (!$isShippingSaveInAddressBook && $isBillingSaveInAddressBook) {
                $customerAddress = $this->_addressFactory->create();

                $address = $this->_addressFactory->create()
                    ->getCollection()
                    ->addFieldToFilter('parent_id', $quote->getCustomerId())
                    ->addOrder('entity_id', 'desc')->getFirstItem();

                $customerAddress->load($address->getId());
                try {

                    $data = json_decode($customBillingAddress);
                    $addressData = $customerAddress->getDataModel();
                    foreach ($data as $item) {
                        $addressData->setCustomAttribute($item->name, $item->value);
                    }
                    $customerAddress->updateData($addressData);
                    $customerAddress->save();
                    $this->updateOrderAddress(
                        $order->getId(),
                        true,
                        $customerAddress->getId(),
                        $order->getCustomerId(),
                        $order->getQuoteId()
                    );
                    $this->_logger->debug('Save success');
                } catch (\Exception $e) {
                    $this->_logger->debug($e->getTraceAsString());
                }
            } elseif (!$isBillingSaveInAddressBook && !$isShippingSaveInAddressBook) {

            } elseif ($isShippingSaveInAddressBook && $isBillingSaveInAddressBook) {
                $shippingAddress = $this->_addressFactory->create();
                $billingAddress = $this->_addressFactory->create();

                $addresses = $this->_addressFactory->create()
                    ->getCollection()
                    ->addFieldToFilter('parent_id', $quote->getCustomerId())
                    ->addOrder('entity_id', 'desc')->setPageSize(2);
                $billingAddress->load($addresses->getFirstItem()->getId());
                $shippingAddress->load($addresses->getLastItem()->getId());

                try {

                    $billingData = json_decode($customBillingAddress);
                    $billingAddressData = $billingAddress->getDataModel();
                    foreach ($billingData as $item) {
                        $billingAddressData->setCustomAttribute($item->name, $item->value);
                    }
                    $billingAddress->updateData($billingAddressData);
                    $billingAddress->save();
                    $this->updateOrderAddress(
                        $order->getId(),
                        true,
                        $billingAddress->getId(),
                        $order->getCustomerId(),
                        $order->getQuoteId()
                    );
                    $shippingData = json_decode($customShippingAddress);
                    $shippingAddressData = $shippingAddress->getDataModel();
                    foreach ($shippingData as $item) {
                        $shippingAddressData->setCustomAttribute($item->name, $item->value);
                    }
                    $shippingAddress->updateData($shippingAddressData);
                    $shippingAddress->save();
                    $this->updateOrderAddress(
                        $order->getId(),
                        false,
                        $shippingAddress->getId(),
                        $order->getCustomerId(),
                        $order->getQuoteId()
                    );
                    $this->_logger->debug('Save success');
                } catch (\Exception $e) {
                    $this->_logger->debug($e->getTraceAsString());
                }
            }
        } elseif ($shippingCustomerAddressId && !$billingCustomerAddressId) {
            if ($isBillingSaveInAddressBook) {

                $customerAddress = $this->_addressFactory->create();

                $address = $this->_addressFactory->create()
                    ->getCollection()
                    ->addFieldToFilter('parent_id', $quote->getCustomerId())
                    ->addOrder('entity_id', 'desc')->getFirstItem();

                $customerAddress->load($address->getId());

                try {

                    $data = json_decode($customBillingAddress);
                    $addressData = $customerAddress->getDataModel();
                    foreach ($data as $item) {
                        $addressData->setCustomAttribute($item->name, $item->value);
                    }
                    $customerAddress->updateData($addressData);
                    $customerAddress->save();
                    $this->updateOrderAddress(
                        $order->getId(),
                        true,
                        $customerAddress->getId(),
                        $order->getCustomerId(),
                        $order->getQuoteId()
                    );
                    $this->_logger->debug('Save success');
                } catch (\Exception $e) {
                    $this->_logger->debug($e->getTraceAsString());
                }
            }
        } elseif (!$shippingCustomerAddressId && $billingCustomerAddressId) {
            if ($isShippingSaveInAddressBook) {
                $customerAddress = $this->_addressFactory->create();

                $address = $this->_addressFactory->create()
                    ->getCollection()
                    ->addFieldToFilter('parent_id', $quote->getCustomerId())
                    ->addOrder('entity_id', 'desc')->getFirstItem();

                $customerAddress->load($address->getId());
                try {

                    $data = json_decode($customShippingAddress);
                    $addressData = $customerAddress->getDataModel();
                    foreach ($data as $item) {
                        $addressData->setCustomAttribute($item->name, $item->value);
                    }
                    $customerAddress->updateData($addressData);
                    $customerAddress->save();
                    $this->updateOrderAddress(
                        $order->getId(),
                        false,
                        $customerAddress->getId(),
                        $order->getCustomerId(),
                        $order->getQuoteId()
                    );
                    $this->_logger->debug('Save success');
                } catch (\Exception $e) {
                    $this->_logger->debug($e->getTraceAsString());
                }
            } elseif( $isBillingSaveInAddressBook ) {
                //Fix for magento2.1.1 issue, if only billing address can be saved as new address.
                $customerAddress = $this->_addressFactory->create();

                $address = $this->_addressFactory->create()
                    ->getCollection()
                    ->addFieldToFilter('parent_id', $quote->getCustomerId())
                    ->addOrder('entity_id', 'desc')->getFirstItem();

                $customerAddress->load($address->getId());
                try {

                    $data = json_decode($customBillingAddress);
                    $addressData = $customerAddress->getDataModel();
                    foreach ($data as $item) {
                        $addressData->setCustomAttribute($item->name, $item->value);
                    }
                    $customerAddress->updateData($addressData);
                    $customerAddress->save();
                    $this->updateOrderAddress(
                        $order->getId(),
                        true,
                        $customerAddress->getId(),
                        $order->getCustomerId(),
                        $order->getQuoteId()
                    );
                    $this->_logger->debug('Save success');
                } catch (\Exception $e) {
                    $this->_logger->debug($e->getTraceAsString());
                }
            }
        } elseif ($shippingCustomerAddressId && $billingCustomerAddressId) {

        }
    }

    /**
     * @param Observer $observer
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        /** @var \Magento\Sales\Api\Data\OrderInterface $order */
        $order = $observer->getOrder();
        $this->_logger->debug('Customer is guest:'.$order->getData('customer_is_guest'));
        if($order->getData('customer_is_guest') != 1) {
            $quoteRepository = $this->_objectManager->create('Magento\Quote\Model\QuoteRepository');
            $quote = $quoteRepository->get($order->getQuoteId());
            $customBillingAddress = $quote->getData('custom_attribute_billing_address_data');
            $customShippingAddress = $quote->getData('custom_attribute_shipping_address_data');
            $shippingCustomerAddressId = $quote->getShippingAddress()->getData('customer_address_id');
            $isShippingSaveInAddressBook = $quote->getShippingAddress()->getData('save_in_address_book');

            $billingCustomerAddressId = $quote->getBillingAddress()->getData('customer_address_id');
            $isBillingSaveInAddressBook = $quote->getBillingAddress()->getData('save_in_address_book');

            $isSameAsBilling = $quote->getShippingAddress()->getData('same_as_billing');
            if(!$this->checkVersion()) {
                $this->saveCustomAttributesForAddressIfVersionFalse(
                    $order,
                    $quote,
                    $customShippingAddress,
                    $customBillingAddress,
                    $shippingCustomerAddressId,
                    $billingCustomerAddressId,
                    $isShippingSaveInAddressBook,
                    $isBillingSaveInAddressBook
                );
            } else {
                $this->saveCustomAttributesForAddressIfVersionTrue(
                    $quote,
                    $customShippingAddress,
                    $customBillingAddress,
                    $shippingCustomerAddressId,
                    $billingCustomerAddressId,
                    $isShippingSaveInAddressBook,
                    $isBillingSaveInAddressBook
                );
            }
        }
    }


}