/**
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */
/*global define,alert*/
define(
    [
        'ko',
        'Magento_Checkout/js/model/quote',
        'Magento_Checkout/js/model/resource-url-manager',
        'mage/storage',
        'Magento_Checkout/js/model/payment-service',
        'Magento_Checkout/js/model/payment/method-converter',
        'Magento_Checkout/js/model/error-processor',
        'Magento_Checkout/js/model/full-screen-loader',
        'Magento_Checkout/js/action/select-billing-address'
    ],
    function (
        ko,
        quote,
        resourceUrlManager,
        storage,
        paymentService,
        methodConverter,
        errorProcessor,
        fullScreenLoader,
        selectBillingAddressAction
    ) {
        'use strict';

        return {
            saveShippingInformation: function () {
                var payload;

                var listInputFields= jQuery('input, select, textarea', 'div[name^="shippingAddress"]:not(._error):not([name$=".firstname"], [name$=".lastname"], [name$=".company"], [name$=".city"], [name$=".region"], [name$=".postcode"], [name$= ".country_id"], [name$=".telephone"], [name$=".region_id"], [name^="shippingAddress.street"])');

                var jsonObj = [];

                listInputFields.each(function(){

                    var name = jQuery(this).attr('name');
                    var value = jQuery(this).val();

                    if(jQuery(this).attr('class') == 'checkbox') {
                        if(jQuery(this).prop('checked')) {
                            value = 1;
                        } else {
                            value = 0;
                        }
                    }

                    var item = {};
                    item['name'] = name;
                    item['value'] = value;

                    jsonObj.push(item);

                });

                console.log(jsonObj);

                var custom_data = JSON.stringify(jsonObj);

                if (!quote.billingAddress()) {
                    selectBillingAddressAction(quote.shippingAddress());
                }

                payload = {
                    addressInformation: {
                        shipping_address: quote.shippingAddress(),
                        billing_address: quote.billingAddress(),
                        shipping_method_code: quote.shippingMethod().method_code,
                        shipping_carrier_code: quote.shippingMethod().carrier_code,
                        extension_attributes: {
                            custom_data: custom_data
                        }
                    }
                };

                fullScreenLoader.startLoader();

                return storage.post(
                    resourceUrlManager.getUrlForSetShippingInformation(quote),
                    JSON.stringify(payload)
                ).done(
                    function (response) {
                        quote.setTotals(response.totals);
                        paymentService.setPaymentMethods(methodConverter(response.payment_methods));
                        fullScreenLoader.stopLoader();
                    }
                ).fail(
                    function (response) {
                        errorProcessor.process(response);
                        fullScreenLoader.stopLoader();
                    }
                );
            }
        };
    }
);
